@@ -5,7 +5,7 @@ import logging
 import uuid
 import warnings
 
-from ..core import BaseClient, LoopHelper
+from ..core import BaseClient, LoopHelper, get_or_create_loop
 from .external_task_worker import ExternalTaskWorker
 
 logger = logging.getLogger(__name__)
@@ -19,7 +19,7 @@ class ExternalTaskClient(BaseClient):
         new_kwargs['on_shutdown'] = self.__on_shutdown
 
         if loop is None:
-            loop = asyncio.get_event_loop()
+            loop = get_or_create_loop()
         else:
             if 'install_signals' not in new_kwargs:
                 new_kwargs['install_signals'] = False
