@@ -5,7 +5,7 @@ from operator import truediv
 import traceback
 import concurrent.futures
 
-from ..core import BaseClient, LoopHelper
+from ..core import BaseClient, ensure_has_loop
 from .functional_error import FunctionalError
 
 
@@ -102,7 +102,12 @@ class ExternalTaskWorker(BaseClient):
                         else:
                             return await self._handler(self._payload)
 
-                    new_loop = asyncio.new_event_loop()
+                    #new_loop = asyncio.new_event_loop()
+                    # see https://stackoverflow.com/questions/46727787/runtimeerror-there-is-no-current-event-loop-in-thread-in-async-apscheduler
+                    #asyncio.set_event_loop(new_loop) 
+                
+                    new_loop = ensure_has_loop()
+
                     result = new_loop.run_until_complete(async_handler())
 
                     return result
@@ -114,11 +119,15 @@ class ExternalTaskWorker(BaseClient):
                 fkt = None
                 if self.__with_external_task_param():
                     def with_all_args():
+                        ensure_has_loop()
+
                         result = self._handler(self._payload, self._external_task)
                         return result
                     fkt = with_all_args
                 else:
                     def with_some_args():
+                        ensure_has_loop()
+                        
                         result = self._handler(self._payload)
                         return result
                     fkt = with_some_args
@@ -130,7 +139,7 @@ class ExternalTaskWorker(BaseClient):
             
         except FunctionalError as fe:
             logger.warning(f"Finish external task with functional error (code: {fe.get_code()}, message: {fe.get_message()})")
-            await self.__finish_task_with_functional_errors(fe.get_code(), fe.get_message())
+            await self.__finish_task_with_functional_errors(fe.get_code(), fe.get_message(), fe.get_details())
 
         except Exception as e:
             formatted_lines = traceback.format_exc().splitlines()
@@ -168,7 +177,7 @@ class ExternalTaskWorker(BaseClient):
         result = await self.do_put(path, payload)
         logger.info(f"finished external task '{external_task_id}' successfully.")
 
-    async def __finish_task_with_functional_errors(self, error_code, error_message):
+    async def __finish_task_with_functional_errors(self, error_code, error_message, error_details=""):
         logger.warn(f"finished external task_with functional errors '{self._external_task}', '{error_code}', '{error_message}'.")
         path = f"/atlas_engine/api/v1/external_tasks/{self._external_task['id']}/handle_bpmn_error"
 
@@ -176,19 +185,21 @@ class ExternalTaskWorker(BaseClient):
             "workerId": self._external_task['workerId'],
             "bpmnError": {
                 "errorCode": error_code,
-                "errorMessage": error_message
+                "errorMessage": error_message,
+                "errorDetails": error_details
             }
         }
 
         await self.do_put(path, payload)
 
-    async def __finish_task_with_technical_errors(self, error_message, error_details):
+    async def __finish_task_with_technical_errors(self, error_message, error_details, error_code=500):
         logger.warn(f"finished task with technical errors '{self._external_task}', '{error_message}', '{error_details}'.")
         path = f"/atlas_engine/api/v1/external_tasks/{self._external_task['id']}/handle_service_error"
 
         payload = {
             "workerId": self._external_task['workerId'],
             "error": {
+                "errorCode": error_code,
                 "errorMessage": error_message,
                 "errorDetails": error_details
             }
